import 'dart:async';
import 'package:flutter/services.dart';
import 'linkv_api_defines.dart';
import 'linkv_video_callback.dart';

class LinkVRTCEngine {
  static const MethodChannel _channel = const MethodChannel('LVRTCEngine');
  static LinkVVideoCallback _videoCallback;

  /// 系统版本号
  static Future<String> get platformVersion async {
    final String version = await _channel.invokeMethod('getPlatformVersion');
    return version;
  }

  /// SDK 内部版本号，汇报给 linkv 时可以提供该版本号
  static Future<String> getSdkVersion() async {
    final String version = await _channel.invokeMethod("getSdkVersion");
    return version;
  }

  /// SDK 鉴权
  /// @param appId 向LinkV申请的appId
  /// @param appSign SDK签名字符串
  /// @param isTestEnv  YES 为使用测试环境，NO 为不使用测试环境（默认为 NO）
  /// @param isInternationalEnv YES 为国际版本，NO 为国内版本（默认为 NO）
  static Future<int> initSDK(String appid, String appSign, bool isTestEnv, bool isInternationalEnv) async {
    final int result = await _channel.invokeMethod("initSDK", {
      'AppId': appid,
      'AppSign': appSign,
      'isTestEnv': isTestEnv,
      'isInternationalEnv':isInternationalEnv
    });
    return result;
  }

  /// 卸载 SDK，销毁单例对象资源
  static void unInitSDK() async {
    await _channel.invokeMethod("unInitSDK");
  }

  /// 设置SDK回调
  static void setVideoCallback(LinkVVideoCallback callback) {
    _videoCallback = callback;
    _channel.setMethodCallHandler(_handleVideoCallback);
  }

  /// 加入音视频房间（音视频会议）
  /// @param roomId 房间 ID
  /// @param userId 用户 ID
  /// @param isHost 是否是主持人
  /// @param isOnlyAudio 是否以音频模式加入
  static Future<LinkvLoginRoomResult> loginRoom(String roomId, String userId, bool isHost, bool isAudioOnly) async {
    final Map<dynamic, dynamic> resultMap = await _channel.invokeMethod("loginRoom", {
      "roomId":roomId,
      "userId":userId,
      "isHost":isHost,
      "isAudioOnly":isAudioOnly
    });
    LinkvLoginRoomResult result = LinkvLoginRoomResult();
    List<LVUser> users = List<LVUser>();
    List<dynamic> nativeList = resultMap["users"];
    if(nativeList != null) {
      for(Map<dynamic,dynamic> item in nativeList) {
        LVUser user = LVUser();
        user.userId = item["userId"];
        List<String> pullUrls = List<String>();
        if(item["pullUrls"] != null) {
          for(var url in item["pullUrls"]) {
            pullUrls.add(url);
          }
        }
        user.pullUrls = pullUrls;
        users.add(user);
      }
    }
    result.code = resultMap["code"];
    result.users = users;
    return result;
  }

  /// 设置推流时数据统计信息回调的频率，回调方法请看 OnPublishQualityUpdate 回调
  /// @param seconds 回调频率
  static void setPublishQualityMonitorCycle(int seconds) async {
    await _channel.invokeMethod("setPublishQualityMonitorCycle", {'seconds':seconds});
  }

  /// 发布音视频数据（推流），如果登录房间时 isOnlyAudio 为 YES 则只发送音频数据，视频数据不发送，推流状态的变更通过 OnPublishStateUpdate: 回调
  static void startPublishing() async {
    await _channel.invokeMethod("startPublishing");
  }

  /// 设置拉流时数据统计信息回调的频率，回调方法请看 OnPlayQualityUpate 回调
  /// @param seconds 回调频率
  static void setPlayQualityMonitor(int seconds) async {

    await _channel.invokeMethod("setPlayQualityMonitor", {"seconds": seconds});
  }

  /// 开始播放一路音视频流，播放状态的变更通过 OnPlayStateUpdate:userId: 回调
  /// @param userId 要播放的音视频流用户 ID
  static void startPlayingStream(String userId) async {
    if (userId == null || userId.length == 0) return;

    await _channel.invokeMethod("startPlayingStream", {"streamId": userId});
  }

  /// 停止播放一路音视频流
  /// @param userId 要停止播放的音视频流用户 ID
  static void stopPlayingStream(String userId) async {
    if (userId == null || userId.length == 0) return;

    await _channel.invokeMethod("stopPlayingStream", {"streamId": userId});
  }

  /// 退出房间
  static void logoutRoom() async {
    await _channel.invokeMethod("logoutRoom");
  }

  /// 停止视频采集
  static void stopCapture() async {
    await _channel.invokeMethod("stopCapture");
  }

  /// 开始视频采集
  static void startCapture() async {
    await _channel.invokeMethod("startCapture");
  }

  /// 停止发布音视频数据
  static void stopPublishStream() async {
    await _channel.invokeMethod("stopPublishStream");
  }

  /// 切换摄像头
  /// @param position 摄像头位置
  static void switchCamera(LVCameraPosition position) async {
    await _channel.invokeMethod("switchCamera", {"cameraPosition": position.index});
  }

  /// 设置视频编码参数
  /// @param config 视频分辨率
  /// @discussion 注：如果后面设置的视频采集分辨率大于之前设置的分辨率，摄像头可能会进行重置操作
  static void setAVConfig(LVAVConfig config) async {
    await _channel.invokeMethod("setAVConfig", {"config": config.index});
  }

  /// 设置美颜
  /// @param beauty 目前包含：美颜、明亮度、饱和度
  /// @param level 取值范围为[0~1]，关闭美颜传-1
  static void setBeautyLevel(LVBeauty beauty, double level) async {
    await _channel.invokeMethod("setBeautyLevel", {"beauty": beauty.index, "level": level});
  }

  // MARK: - Video Callback
  static Future<dynamic> _handleVideoCallback(MethodCall call) {
    if (call.method == "onDrawFrame") {
      _videoCallback?.onDrawFrame(0, 0, 0, "s", "s1");
    } else if (call.method == "onAddRemoter") {
      LVUser user = LVUser();
      user.userId = call.arguments["userId"];
      if(call.arguments["pullUrls"] != null) {
        List<String> pullUrls = List<String>();
        for(var url in call.arguments["pullUrls"]) {
          pullUrls.add(url);
        }
        user.pullUrls = pullUrls;
      }
      print("D/Wing LinkVRTCEngine onAddRemoter = " + user.userId);
      _videoCallback?.onAddRemoter(user);
    } else if (call.method == "onDeleteRemoter") {
        _videoCallback?.onDeleteRemoter(call.arguments["streamId"]);
    } else if (call.method == "onKickOff") {
      _videoCallback?.onKickOff(call.arguments["state"] ,call.arguments["streamId"]);
    } else if (call.method == "onRoomDisconnect") {
      _videoCallback?.onRoomDisconnect(call.arguments["state"]);
    } else if (call.method == "onRoomReconnect") {
      _videoCallback?.onRoomReconnect();
    } else if (call.method == "onMixComplete") {
      _videoCallback?.onMixComplete(call.arguments["b"]);
    } else if (call.method == "onPublishQualityUpdate") {
      LVVideoStatistic videoStatistic = LVVideoStatistic.convertFromMap(call.arguments);
      _videoCallback?.onPublishQualityUpdate(videoStatistic);
    } else if (call.method == "onPublishStateUpdate") {
      _videoCallback?.onPublishStateUpdate(call.arguments["state"]);
    } else if (call.method == "onPlayQualityUpdate") {
      LVVideoStatistic videoStatistic = LVVideoStatistic.convertFromMap(call.arguments);
      _videoCallback?.onPlayQualityUpdate(videoStatistic, call.arguments["uid"]);
    } else if (call.method == "onPlayStateUpdate") {
      _videoCallback?.onPlayStateUpdate(call.arguments["state"], call.arguments["streamId"]);
    } else if (call.method == "onExitRoomComplete") {
      _videoCallback?.onExitRoomComplete();
    }

    return Future.value(true);
  }

}
